﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Joseph Helm and Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.BinPacking.Interfaces;

namespace HeuristicLab.Encodings.PackingEncoding.MultiComponentVector {
  [Item("ChangePositionMove", "A move on a multi component vector that is specified by two indexes. The packing informations at the first index is inserted at the second one.")]
  [StorableClass]
  public class ChangePositionMove : MultiComponentVectorMove {
    [Storable]
    public int AffectedGroup { get; protected set; }
    [Storable]
    public int Index { get; protected set; }
    [Storable]
    public int TargetIndex { get; protected set; }

    [StorableConstructor]
    protected ChangePositionMove(bool deserializing) : base(deserializing) { }
    protected ChangePositionMove(ChangePositionMove original, Cloner cloner)
      : base(original, cloner) {
      this.AffectedGroup = original.AffectedGroup;
      this.Index = original.Index;
      this.TargetIndex = original.TargetIndex;
    }
    public ChangePositionMove(int affectedBin, int index, int targetIndex, MultiComponentVectorEncoding multiComponentVector)
      : base(multiComponentVector) {
        AffectedGroup = affectedBin;
        Index = index;
        TargetIndex = targetIndex;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ChangePositionMove(this, cloner);
    }

    public override IPackingSolutionEncoding GetSolutionAfterMove() {
      return GetVectorAfterMove(MultiComponentVector, AffectedGroup, Index, TargetIndex);
    }
    public static MultiComponentVectorEncoding GetVectorAfterMove(MultiComponentVectorEncoding multiComponentVector, int affectedBin, int index, int targetIndex) {
      var result = multiComponentVector.Clone(new Cloner()) as MultiComponentVectorEncoding;
      var aux = result.PackingInformations[affectedBin][index];
      result.PackingInformations[affectedBin].Remove(aux);
      result.PackingInformations[affectedBin].Insert(targetIndex, aux);
      return result;
    }

    public override System.Type GetMoveAttributeType() {
      return typeof(ChangePositionMoveAttribute);
    }

    public override string ToString() {
      return "PM(g=" + AffectedGroup  + ",i=" + Index + ",ti=" + TargetIndex + ")";
    }

    public override MultiComponentVectorMoveAttribute GetAttribute(double quality) {
      return new ChangePositionMoveAttribute(AffectedGroup, Index,
        MultiComponentVector.PackingInformations[AffectedGroup][Index].ItemID,
        TargetIndex, quality);
    }

    public override bool BelongsToAttribute(MultiComponentVectorMoveAttribute attribute, bool hardCriterion) {
      ChangePositionMoveAttribute actualAttribute = attribute as ChangePositionMoveAttribute;
      if (actualAttribute != null) {
        if (hardCriterion) {
          if (Index == actualAttribute.Index
          || TargetIndex == actualAttribute.TargetIndex
          || MultiComponentVector.PackingInformations[AffectedGroup][Index].ItemID == actualAttribute.ItemID)
            return true;
        } else {
          if (Index == actualAttribute.Index
          && TargetIndex == actualAttribute.TargetIndex
          && MultiComponentVector.PackingInformations[AffectedGroup][Index].ItemID == actualAttribute.ItemID)
            return true;
        }
      }

      return false;
    }
  }
}